import * as React from 'react';
import { range } from 'ramda';
import * as moment from 'moment';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';

import { IHTMLElement } from '@romanjaros/treejs-types/form';

import Section from '../../Section';
import Button, { ButtonTypes } from '../../Button';

import { MONTHS, WEEK_DAYS_SHORT } from '../constants';
import {
	StyledCalendar,
	StyledCalendarHeader,
	StyledCalendarNavigation,
	StyledCalendarRow,
	StyledCalendarNavigationMonth,
} from '../style';


interface IProps extends IHTMLElement<string> {
	format?: string;
	value?: string;
}

interface IState {
	currentMoment: moment.Moment;
	daysInMonth: number;
	firstDayOfMonthAsDayIndex: number;
	value?: string;
}

const todayMoment: Date = new Date();

class Calendar extends React.Component<IProps, IState> {
	public static defaultProps = {
		format: 'DD.MM.YYYY',
	};

	public constructor(props: IProps) {
		super(props);

		let defaultMoment: moment.Moment = moment();
		if (moment(props.value, props.format).isValid()) {
			defaultMoment = moment(props.value, props.format);
		}

		this.state = {
			value: props.value,
			currentMoment: defaultMoment,
			firstDayOfMonthAsDayIndex: defaultMoment.startOf('month').day() - 1,
			daysInMonth: defaultMoment.daysInMonth(),
		}
	}

	public componentDidUpdate(prevProps: IProps, prevState: IState): void {
		if (prevState.value !== this.props.value) {
			this.setState({
				value: this.props.value,
			});
		}
	}

	private getStartOfWeek = (row = 0) => {
		return (7 - this.state.firstDayOfMonthAsDayIndex) + ((row - 1) * 7);
	};

	private getWeeksInMonth = () => {
		return (this.state.daysInMonth + this.state.firstDayOfMonthAsDayIndex - 1) / 7;
	};

	private isToday = (day: number): boolean => {
		return moment(todayMoment).date() === day
			&& moment(todayMoment).month() === this.state.currentMoment.month()
			&& moment(todayMoment).year() === this.state.currentMoment.year()
	};

	private isSelected = (day: number): boolean => {
		if (moment(this.state.value, this.props.format).isValid()) {
			return moment(this.state.value, this.props.format).date() === day
				&& moment(this.state.value, this.props.format).month() === this.state.currentMoment.month()
				&& moment(this.state.value, this.props.format).year() === this.state.currentMoment.year()
		}
		return false;
	};

	private getHeader = (): React.ReactElement[] => (
		WEEK_DAYS_SHORT.map(
			(day, i) => {
				return (
					<StyledCalendarHeader key={i}>
						{day}
					</StyledCalendarHeader>
				);
			}
		)
	);

	private getRow = (row = 0): React.ReactElement[] => {
		let day = 0;
		if (row !== 0) {
			day = this.getStartOfWeek(row);
		}
		return range(0, 7).map(
			(i) => {
				if ((this.state.firstDayOfMonthAsDayIndex <= i && row === 0) || (this.state.daysInMonth > day && row !== 0)) {
					day++;
					return (
						<StyledCalendarRow
							key={i}
							hasValue
							isToday={this.isToday(day)}
							isSelected={this.isSelected(day)}
							onClick={this.handleChangeDate(day)}
						>
							{day}
						</StyledCalendarRow>
					)
				} else {
					return (
						<StyledCalendarRow key={i} hasValue={false} isToday={false} isSelected={false} />
					);
				}
			}
		);
	};

	private getNavigationHeader = () => (
		<React.Fragment>
			<StyledCalendarNavigation
				colSpan={1}
				onClick={this.handleClickPrevYear}
			>
				<FontAwesomeIcon icon="angle-double-left" />
			</StyledCalendarNavigation>
			<StyledCalendarNavigation
				colSpan={1}
				onClick={this.handleClickPrevMonth}
			>
				<FontAwesomeIcon icon="angle-left" />
			</StyledCalendarNavigation>
			<StyledCalendarNavigationMonth
				colSpan={3}
			>
				<div>{this.state.currentMoment.year()}</div>
				<div><b>{MONTHS[this.state.currentMoment.month()]}</b></div>
			</StyledCalendarNavigationMonth>
			<StyledCalendarNavigation
				colSpan={1}
				onClick={this.handleClickNextMonth}
			>
				<FontAwesomeIcon icon="angle-right" />
			</StyledCalendarNavigation>
			<StyledCalendarNavigation
				colSpan={1}
				onClick={this.handleClickNextYear}
			>
				<FontAwesomeIcon icon="angle-double-right" />
			</StyledCalendarNavigation>
		</React.Fragment>
	);

	private setDate = (date: moment.Moment) => {
		this.setState({
			currentMoment: date,
			firstDayOfMonthAsDayIndex: date.startOf('month').day() - 1,
			daysInMonth: date.daysInMonth(),
		})
	};

	private handleClickSetToday = () => {
		this.setDate(moment(todayMoment));
	};

	private handleClickPrevMonth = () => {
		this.setDate(this.state.currentMoment.subtract(1, 'months'))
	};

	private handleClickPrevYear = () => {
		this.setDate(this.state.currentMoment.subtract(1, 'years'))
	};

	private handleClickNextMonth = () => {
		this.setDate(this.state.currentMoment.add(1, 'months'))
	};

	private handleClickNextYear = () => {
		this.setDate(this.state.currentMoment.add(1, 'years'))
	};

	private handleChangeDate = (day: number) => (): void => {
		const date = this.state.currentMoment.set('date', day).format(this.props.format);
		if (this.props.onChange instanceof Function) {
			this.props.onChange(date, null);
		}
	};

	public render(): React.ReactElement {
		return (
			<div>
				<StyledCalendar>
					<thead>
						<tr>{this.getNavigationHeader()}</tr>
						<tr>{this.getHeader()}</tr>
					</thead>
					<tbody>
						{
							range(0, this.getWeeksInMonth()).map(
								(i: number) => <tr key={i}>{this.getRow(i)}</tr>
							)
						}
					</tbody>
				</StyledCalendar>
				<Section>
					<Button
						title="Zobrazit dnešní datum"
						status={ButtonTypes.info}
						onClick={this.handleClickSetToday}
					/>
				</Section>
			</div>
		);
	}
}

export default Calendar;
