import * as React from 'react';
import { findIndex, propEq, propOr } from 'ramda';

import Section from '../../Section';
import Button, { ButtonTypes } from '../../Button';
import { IModalId } from '../../Modal/types';

import { IOption } from '../types';
import { StyledSelectBoxOptions } from '../style';
import { isNilOrEmpty } from '@romanjaros/treejs-utils';


interface IProps {
	closeModal: (id: IModalId) => void;
	name: IModalId;
	onChange: (options: IOption) => void;
	onRemove: () => void;
	options: IOption[];
	selectedOption: IOption;
}

interface IState {
	index: number;
}

class Modal extends React.Component<IProps, IState> {

	public readonly state = {
		index: 0,
	};

	public componentDidMount(): void {
		window.addEventListener("keydown", this.handleKeyUp, { passive: false });

		if (!isNilOrEmpty(this.props.options)) {
			if (!isNilOrEmpty(this.props.selectedOption)) {
				const index = findIndex(propEq('code', this.props.selectedOption.code), this.props.options);
				if (index !== -1) {
					this.setState({
						index,
					})
				}
			}
		}
	}

	public componentWillUnmount(): void {
		window.removeEventListener("keydown", this.handleKeyUp);
	}

	private setCurrentOption = (optionIndex: number) => (): void => {
		this.setState({
			index: optionIndex,
		});
	};

	private setActiveOption = (option: IOption) => (): void => {
		this.props.onChange(option);
		this.props.closeModal(this.props.name);
	};

	private removeActiveOption = (): void => {
		this.props.onRemove();
		this.props.closeModal(this.props.name);
	};

	private handleKeyUp = (event: KeyboardEvent): void => {
		event.preventDefault();
		let newIndex = this.state.index;

		switch (event.key) {
			case 'ArrowDown':
				newIndex += 1;
				break;
			case 'ArrowUp':
				newIndex -= 1;
				break;
			case 'Enter':
				this.setActiveOption(this.props.options[this.state.index])();
				return;
		}

		if (newIndex < 0) {
			newIndex = 0;
		} else if (newIndex > this.props.options.length - 1) {
			newIndex = this.props.options.length - 1;
		}

		this.setCurrentOption(newIndex)();
	};

	public render(): React.ReactElement {
		const {
			options,
		} = this.props;

		return (
			<div>
				{
					options.map(
						({ code, name }, i) => (
							<StyledSelectBoxOptions
								key={code}
								isActive={this.state.index === i}
								isSelected={propOr('', 'code', this.props.selectedOption) === code}
								onClick={this.setActiveOption({ code, name })}
								onMouseEnter={this.setCurrentOption(i)}
							>
								{name}
							</StyledSelectBoxOptions>
						)
					)
				}
				<Section>
					<Button
						title="Odebrat zvolenou hodnotu"
						status={ButtonTypes.error}
						onClick={this.removeActiveOption}
					/>
				</Section>
			</div>
		);
	}
}

export default Modal;
